@@ -30,11 +30,14 @@
 # include <stdint.h>
 #endif
 
-#include "eina_hash.h"
+#include "eina_config.h"
+#include "eina_private.h"
 #include "eina_rbtree.h"
 #include "eina_error.h"
-#include "eina_private.h"
+
+/* undefs EINA_ARG_NONULL() so NULL checks are not compiled out! */
 #include "eina_safety_checks.h"
+#include "eina_hash.h"
 
 /*============================================================================*
  *                                  Local                                     *
@@ -135,8 +138,6 @@ struct _Eina_Hash_Each
    const void *data;
 };
 
-static int _eina_hash_init_count = 0;
-
 #undef get16bits
 #if (defined(__GNUC__) && defined(__i386__)) || defined(__WATCOMC__) \
   || defined(_MSC_VER) || defined (__BORLANDC__) || defined (__TURBOC__)
@@ -420,7 +421,7 @@ _eina_hash_del_by_key(Eina_Hash *hash, const void *key, const void *data)
    if (!key) return EINA_FALSE;
    if (!hash->buckets) return EINA_FALSE;
 
-   key_length = hash->key_length_cb(key);
+   key_length = hash->key_length_cb ? hash->key_length_cb(key) : 0;
    key_hash = hash->key_hash_cb(key, key_length);
    return _eina_hash_del_by_key_hash(hash, key, key_length, key_hash, data);
 }
@@ -439,6 +440,13 @@ _eina_string_key_cmp(const char *key1, __UNUSED__ int key1_length,
    return strcmp(key1, key2);
 }
 
+static int
+_eina_stringshared_key_cmp(const char *key1, __UNUSED__ int key1_length,
+			   const char *key2, __UNUSED__ int key2_length)
+{
+   return key1 - key2;
+}
+
 static unsigned int
 _eina_int32_key_length(__UNUSED__ const uint32_t *key)
 {
@@ -449,11 +457,7 @@ static int
 _eina_int32_key_cmp(const uint32_t *key1, __UNUSED__ int key1_length,
 		    const uint32_t *key2, __UNUSED__ int key2_length)
 {
-  if (*key1 > *key2)
-    return 1;
-  if (*key1 < *key2)
-    return -1;
-  return 0;
+   return *key1 - *key2;
 }
 
 static unsigned int
@@ -466,14 +470,9 @@ static int
 _eina_int64_key_cmp(const uint64_t *key1, __UNUSED__ int key1_length,
 		    const uint64_t *key2, __UNUSED__ int key2_length)
 {
-  if (*key1 > *key2)
-    return 1;
-  if (*key1 < *key2)
-    return -1;
-  return 0;
+   return *key1 - *key2;
 }
 
-
 static Eina_Bool
 _eina_foreach_cb(const Eina_Hash *hash, Eina_Hash_Tuple *data, Eina_Hash_Foreach_Data *fdata)
 {
@@ -580,7 +579,7 @@ _eina_hash_iterator_next(Eina_Iterator_Hash *it, void **data)
    it->index++;
    it->bucket = bucket;
 
-   if (ok && data)
+   if (ok)
      *data = it->get_content(it);
 
    return ok;
@@ -614,18 +613,6 @@ _eina_hash_iterator_free(Eina_Iterator_Hash *it)
  *                                   API                                      *
  *============================================================================*/
 
-/**
- * @addtogroup Eina_Data_Types_Group Data Types
- *
- * @{
- */
-
-/**
- * @addtogroup Eina_Containers_Group Containers
- *
- * @{
- */
-
 /**
  * @addtogroup Eina_Hash_Group Hash Table
  *
@@ -651,48 +638,6 @@ _eina_hash_iterator_free(Eina_Iterator_Hash *it)
  * @{
  */
 
-/**
- * @brief Initialize the hash table module.
- *
- * @return 1 or greater on success, 0 on error.
- *
- * This function just sets up the error module or Eina. It is also
- * called by eina_init(). It returns 0 on failure, otherwise it
- * returns the number of times eina_error_init() has already been
- * called.
- */
-EAPI int
-eina_hash_init(void)
-{
-   if (!_eina_hash_init_count)
-     if (!eina_error_init())
-       {
-	  fprintf(stderr, "Could not initialize eina error module\n");
-	  return 0;
-       }
-
-   return ++_eina_hash_init_count;
-}
-
-/**
- * @brief Shut down the hash table module.
- *
- * @return 0 when the error module is completely shut down, 1 or
- * greater otherwise.
- *
- * This function just shut down the error module set up by
- * eina_hash_init(). It is also called by eina_shutdown(). It returns
- * 0 when it is called the same number of times than
- * eina_error_init().
- */
-EAPI int
-eina_hash_shutdown(void)
-{
-   if (_eina_hash_init_count == 1)
-     eina_error_shutdown();
-   return --_eina_hash_init_count;
-}
-
 EAPI Eina_Hash *
 eina_hash_new(Eina_Key_Length key_length_cb,
 	      Eina_Key_Cmp key_cmp_cb,
@@ -704,7 +649,6 @@ eina_hash_new(Eina_Key_Length key_length_cb,
    Eina_Hash *new;
 
    eina_error_set(0);
-   EINA_SAFETY_ON_NULL_RETURN_VAL(key_length_cb, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(key_cmp_cb, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(key_hash_cb, NULL);
    EINA_SAFETY_ON_TRUE_RETURN_VAL(buckets_power_size < 3, NULL);
@@ -800,6 +744,16 @@ eina_hash_pointer_new(Eina_Free_Cb data_free_cb)
 #endif
 }
 
+EAPI Eina_Hash *
+eina_hash_stringshared_new(Eina_Free_Cb data_free_cb)
+{
+   return eina_hash_new(NULL,
+			EINA_KEY_CMP(_eina_stringshared_key_cmp),
+			EINA_KEY_HASH(eina_hash_superfast),
+			data_free_cb,
+			EINA_HASH_BUCKET_SIZE);
+}
+
 /**
  * Retrieves the number of buckets available in the given hash table.
  * @param hash The given hash table.
@@ -819,7 +773,8 @@ eina_hash_population(const Eina_Hash *hash)
  * @param hash The hash table to be freed
  *
  * This function frees up all the memory allocated to storing the specified
- * hash tale pointed to by @p hash. Any entries in the table that the program
+ * hash tale pointed to by @p hash. If no data_free_cb has been passed to the
+ * hash at creation time, any entries in the table that the program
  * has no more pointers for elsewhere may now be lost, so this should only be
  * called if the program has lready freed any allocated data in the hash table
  * or has the pointers for data in teh table stored elswehere as well.
@@ -941,12 +896,11 @@ eina_hash_add(Eina_Hash *hash, const void *key, const void *data)
 
    EINA_MAGIC_CHECK_HASH(hash);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash, EINA_FALSE);
-   EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_length_cb, EINA_FALSE);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_hash_cb, EINA_FALSE);
    EINA_SAFETY_ON_NULL_RETURN_VAL(key, EINA_FALSE);
    EINA_SAFETY_ON_NULL_RETURN_VAL(data, EINA_FALSE);
 
-   key_length = hash->key_length_cb(key);
+   key_length = hash->key_length_cb ? hash->key_length_cb(key) : 0;
    key_hash = hash->key_hash_cb(key, key_length);
 
    return eina_hash_add_by_hash(hash, key, key_length, key_hash, data);
@@ -980,12 +934,11 @@ eina_hash_direct_add(Eina_Hash *hash, const void *key, const void *data)
 
    EINA_MAGIC_CHECK_HASH(hash);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash, EINA_FALSE);
-   EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_length_cb, EINA_FALSE);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_hash_cb, EINA_FALSE);
    EINA_SAFETY_ON_NULL_RETURN_VAL(key, EINA_FALSE);
    EINA_SAFETY_ON_NULL_RETURN_VAL(data, EINA_FALSE);
 
-   key_length = hash->key_length_cb(key);
+   key_length = hash->key_length_cb ? hash->key_length_cb(key) : 0;
    key_hash = hash->key_hash_cb(key, key_length);
 
    return eina_hash_direct_add_by_hash(hash, key, key_length, key_hash, data);
@@ -1167,11 +1120,10 @@ eina_hash_find(const Eina_Hash *hash, const void *key)
    if (!hash) return NULL;
 
    EINA_MAGIC_CHECK_HASH(hash);
-   EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_length_cb, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_hash_cb, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(key, NULL);
 
-   key_length = hash->key_length_cb(key);
+   key_length = hash->key_length_cb ? hash->key_length_cb(key) : 0;
    hash_num = hash->key_hash_cb(key, key_length);
 
    return eina_hash_find_by_hash(hash, key, key_length, hash_num);
@@ -1232,12 +1184,11 @@ eina_hash_modify(Eina_Hash *hash, const void *key, const void *data)
 
    EINA_MAGIC_CHECK_HASH(hash);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash, NULL);
-   EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_length_cb, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(hash->key_hash_cb, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(key, NULL);
    EINA_SAFETY_ON_NULL_RETURN_VAL(data, NULL);
 
-   key_length = hash->key_length_cb(key);
+   key_length = hash->key_length_cb ? hash->key_length_cb(key) : 0;
    hash_num = hash->key_hash_cb(key, key_length);
 
    return eina_hash_modify_by_hash(hash, key, key_length, hash_num, data);
@@ -1502,11 +1453,3 @@ eina_hash_superfast(const char *key, int len)
 /**
  * @}
  */
-
-/**
- * @}
- */
-
-/**
- * @}
- */
