@@ -23,10 +23,13 @@
 #include <stdlib.h>
 #include <assert.h>
 
-#include "eina_inlist.h"
-#include "eina_error.h"
+#include "eina_config.h"
 #include "eina_private.h"
+#include "eina_error.h"
+
+/* undefs EINA_ARG_NONULL() so NULL checks are not compiled out! */
 #include "eina_safety_checks.h"
+#include "eina_inlist.h"
 
 /* FIXME: TODO please, refactor this :) */
 
